searchNodes=[{"ref":"Mix.Tasks.ZigDoc.html","title":"Mix.Tasks.ZigDoc","type":"task","doc":"Used to generate documentation from your zig code and incorporate it into your Elixir documentation. Generally speaking, zig structs will be documented as Elixir modules and zig functions will be incorporated as Elixir functions. If you want to make the zig docgen process override the Elixir mix docs task, add this to your mix.exs: def project do [ ... aliases: [docs: &quot;zig_doc&quot;], ... ] end"},{"ref":"Mix.Tasks.Zigler.GetZig.html","title":"Mix.Tasks.Zigler.GetZig","type":"task","doc":"gets a zig version and puts it into the zigler directory under the subdirectory zig. usage mix zigler.get_zig &lt;version&gt; version can be &quot;latest&quot;"},{"ref":"Mix.Tasks.Zigler.GetZig.html#download_zig_tarball/2","title":"Mix.Tasks.Zigler.GetZig.download_zig_tarball/2","type":"function","doc":""},{"ref":"Mix.Tasks.Zigler.GetZig.html#latest_version/0","title":"Mix.Tasks.Zigler.GetZig.latest_version/0","type":"function","doc":"queries the zig website to obtain the latest version of zig. Only performed at compile time."},{"ref":"Zigler.html","title":"Zigler","type":"module","doc":"Inline NIF support for Zig Motivation Zig is a general-purpose programming language designed for robustness, optimality, and maintainability. The programming philosophy of Zig matches up nicely with the programming philosophy of the BEAM VM and in particular its emphasis on simplicity and structure should very appealing to the practitioners of Elixir. The following features make Zig extremely amenable to inline language support in a BEAM language: simplicity. Zig&#39;s syntax is definable in a simple YACC document and Zig takes a stance against making its featureset more complex (though it may evolve somewhat en route to 1.0) Composability. Zig is unopinionated about how to go about memory allocations. Its allocator interface is very easily able to be backed by the BEAM&#39;s, which means that you have access to generic memory allocation strategies through its composable allocator scheme. C integration. It&#39;s very easy to design C-interop between Zig and C. In fact, Zig is likely to be an easier glue language for C ABIs than C. OOM safety and sanity. Zig makes it easy to defer memory destruction, meaning you often don&#39;t have to worry about memory leaks. By design, alloc should be failable (as is the BEAM&#39;s alloc) so it offers an extra layer of protection for you BEAM VM against an OOM situation. Basic usage In the BEAM, you can define a NIF by consulting the following document and implementing the appropriate shared object/DLL callbacks. However, Zigler will take care of all of this for you. Simply use Zigler in your module, providing the app atom in the property list. Then, use the sigil_Z/2 macro and write zig code. Any nifs you define should be preceded with the /// nif: function_name/arity zig docstring. Example defmodule MyModule do use Zigler, app: :my_app ~Z&quot;&quot;&quot; /// nif: my_func/1 fn my_func(val: i64) i64 { return val + 1; } &quot;&quot;&quot; end Zigler will automatically fill out the appropriate NIF C template, compile the shared object, and bind it into the module pre-compilation. In the case of the example, there will be a MyModule.my_func/1 function call found in the module. Zigler will also make sure that your statically-typed Zig data are guarded when you marshal it from the dynamically-typed BEAM world. However, you may only pass in and return certain types (though the generic beam.term type) is supported. Environment Sometimes, you will need to pass the BEAM environment (which is the code execution context, including process info, etc.) into the NIF function. In this case, you should pass it as the first parameter, as a beam.env type value. Example defmodule MyModule do use Zigler, app: :my_app ~Z&quot;&quot;&quot; /// nif: my_func_with_env/1 fn my_func_with_env(env: beam.env, pid: beam.pid) void { var sendable_term: []u64 = &quot;ping&quot;[0..]; var msg = beam.make_slice(env, sendable_term); var res = e.enif_send(env, pid, env, msg); } &quot;&quot;&quot; end External Libraries If you need to bind static (*.a) or dynamic (*.so) libraries into your module, you may link them with the :libs parameter. Note that for shared libraries, a library with an identical path must exist in the target release environment. Example defmodule Blas do use Zigler, app: :my_app, libs: [&quot;/usr/lib/x86_64-linux-gnu/blas/libblas.so&quot;], include: [&quot;/usr/include/x86_64-linux-gnu&quot;] ~Z&quot;&quot;&quot; const blas = @cImport({ @cInclude(&quot;cblas.h&quot;); ... Compilation assistance If something should go wrong, Zigler will translate the Zig compiler error into an Elixir compiler error, and let you know exactly which line in the ~Z block it came from. Syntactic Sugar Some of the erlang nif terms can get unwieldy, especially in Zig, which prefers terseness. Each of the basic BEAM types is shadowed by a Zig type in the beam module. The beam struct is always imported into the header of the zig file used, so all zig code in the same directory as the module should have access to the beam struct if they @import(&quot;beam.zig&quot;) Importing files If you need to write code outside of the basic module (you will, for anything non-trivial), just place it in the same directory as your module. Example ~Z&quot;&quot;&quot; const extra_code = @import(&quot;extra_code.zig&quot;); /// nif: use_extra_code/1 fn use_extra_code(val: i64) i64 { return extra_code.extra_fn(val); } &quot;&quot;&quot; If you would like to include a custom c header file, create an include/ directory inside your path tree and it will be available to zig as a default search path as follows: ~Z&quot;&quot;&quot; const c = @cImport({ @cInclude(&quot;my_c_header.h&quot;); }); // nif: my_nif/1 ... &quot;&quot;&quot; Documentation Use the builtin zig /// docstring to write your documentation. If it&#39;s in front of the nif declaration, it will wind up in the correct place in your elixir documentation. See Zigler.Doc for more information on how to document in zig and what to document. See Mix.Tasks.ZigDoc for information on how to get your Elixir project to incorporate zig documentation. Tests Use the builtin zig test keyword to write your internal zig unit tests. These can be imported into an ExUnit module by following this example: defmodule MyTest do use ExUnit.Case use Zigler.Unit zigtest ModuleWithZigCode end See Zigler.Unit for more information."},{"ref":"Zigler.html#sigil_Z/2","title":"Zigler.sigil_Z/2","type":"macro","doc":"Analyzes Zig code inline, then assembles a series of code files, stashes them in a temporary directory, compiles it with NIF adapters, and then binds it into the current module. You may have multiple sigil_Z blocks in a single Elixir module if you wish."},{"ref":"Zigler.Doc.html","title":"Zigler.Doc","type":"module","doc":"Translates the docstrings from your Zig code into Elixir documentation. For instructions on how to incorporate this into an Elixir project, consult Mix.Tasks.ZigDoc Documentation forms Currently, Zigler recognizes four types of code segments which should be documented. functions types values errors Functions functions have the following signature: pub fn &lt;identifier&gt;(&lt;parameters&gt;) &lt;type&gt; { and may have the property of being comptime which is due to either the function being itself a comptime function or it having a comptime parameter. NB only pub functions are documented in Zigler, following the Elixir philosophy that only public functions should be documented. Types types have the following signature: pub const &lt;identifier&gt;=&lt;value&gt;; Values values can have one of the following signatures: pub const &lt;identifier&gt;=&lt;value&gt;; pub var &lt;identifier&gt;=&lt;value&gt;; Note that the constant value form is indistinguishable from the type form without doing a full parse and evaluation of the Zig code. In order to avoid doing this, to disambiguate between the two, you must prepend constant value docststrings with a !value token. Example /// !value /// /// a constant representing the value 47. pub const fortyseven = 47; Errors errors appear inside special error struct and should be documented at the per-value level: Example /// docstring for this error struct (if desired) pub const my_error = error { /// docstring for ErrorEnum1 ErrorEnum1, /// docstring for ErrorEnum2 ErrorEnum2 } Scope This module will generate documentation from all Zig code that resides in the same code directory as the base module (or overridden directory, if applicable). Zig code in subdirectories will not be subjected to document generation."},{"ref":"Zigler.Unit.html","title":"Zigler.Unit","type":"module","doc":"Hooks your zig code into ExUnit, by converting zig tests into ExUnit tests. Usage Example Inside your zig code (dependent.zig): const beam = @import(&quot;beam.zig&quot;); const assert = beam.assert; fn one() i64 { return 1; } test &quot;the one function returns one&quot; { assert(one() == 1); } Inside your elixir code: defmodule MyZigModule do use Zigler, app: :my_app ~Z&quot;&quot;&quot; const dependent = @import(&quot;dependent.zig&quot;); /// nif: some_nif_fn/1 ... &quot;&quot;&quot; end Inside your test module: defmodule MyZigTest do use ExUnit.Case, async: true use Zigler.Unit zigtest MyZigModule end Scope This module will run tests in all zig code that resides in the same code directory as the base module (or overridden directory, if applicable). Zig code in subdirectories will not be subjected to test conversion, so if you would like to run a subset of tests using the Zig test facility (and without the support of a BEAM VM), you should put them in subdirectories."},{"ref":"Zigler.Unit.html#zigtest/1","title":"Zigler.Unit.zigtest/1","type":"macro","doc":"loads a module that wraps a zigler NIF, consults the corresponding zig code, generating the corresponding zig tests. Must be called from within a module that has run use ExUnit.Case."},{"ref":"beam.html","title":"beam","type":"module","doc":"This struct contains adapters designed to facilitate interfacing the BEAM&#39;s c-style helpers for NIFs with a more idiomatic Zig-style of programming, for example, the use of slices instead of null-terminated arrays as strings. This struct derives from zig/beam/beam.zig, and you may import it into your module&#39;s zig code by calling: const beam = @import(&quot;beam.zig&quot;) This is done automatically for you inside your ~Z forms, so do NOT use this import statement with inline Zig. Features The BEAM Allocator Wraps e.enif_alloc and e.enif_free functions into a compliant Zig allocator struct. You should thus be able to supply Zig standard library functions which require an allocator a struct that is compliant with its requirements. This is, in particular, useful for slice generation. Example (slice generation) beam = @import(&quot;beam.zig&quot;); fn make_a_slice_of_floats() ![]f32 { return beam.allocator.alloc(f32, 100); } Beacuse Zig features composable allocators, you can very easily implement custom allocators on top of the existing BEAM allocator. Getters Erlang&#39;s NIF interface provides a comprehensive set of methods to retrieve data out of BEAM terms. However, this set of methods presents an error handling scheme that is designed for C and inconsistent with the idiomatic scheme used for Zig best practices. A series of get functions is provided, implementing these methods in accordance to best practices. These include get/3, which is the generic method for getting scalar values, get_X, which are typed methods for retrieving scalar values, and get_slice_of/3, which is the generic method for retrieving a Zig slice from a BEAM list. Naturally, for all of these functions, you will have to provide the BEAM environment value. Examples const beam = @import(&quot;beam.zig&quot;); fn double_value(env: beam.env, value: beam.term) !f64 { return (try beam.get_f64(env, value)) * 2; } fn sum_float_list(env: beam.env, list: beam.term) !f64 { zig_list: []f64 = try beam.get_slice_of(f64, env, list); defer beam.allocator.free(zig_list); // don&#39;t forget to clean up! result: f64 = 0; for (list) |item| { result += item; } return result; } Makers A series of &quot;make&quot; functions is provided which allow for easy export of Zig values back to the BEAM. Typically, these functions are used in the automatic type marshalling performed by Zigler, however, you may want to be able to use them yourself to assemble BEAM datatypes not directly supported by Zig. For example, a custom tuple value. Example const beam = @import(&quot;beam.zig&quot;); const ok_slice=&quot;ok&quot;[0..]; fn to_ok_tuple(env: beam.env, value: i64) !beam.term { tuple_slice: []term = try beam.allocator.alloc(beam.term, 2); tuple_slice[0] = beam.make_atom(env, ok_slice); tuple_slice[1] = beam.make_i64(env, value); return beam.make_tuple(tuple_slice); }"},{"ref":"beam.html#get/3","title":"beam.get/3","type":"function","doc":"A helper for marshalling values from the BEAM runtime into Zig. Use this function if you need support for Zig generics. Used internally to typcheck values coming into Zig slice. supported types: c_int c_long isize usize u8 i32 i64 f16 f32 f64"},{"ref":"beam.html#get_c_int/2","title":"beam.get_c_int/2","type":"function","doc":"Takes a BEAM int term and returns a c_int value. Should only be used for C interop with Zig functions. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_c_long/2","title":"beam.get_c_long/2","type":"function","doc":"Takes a BEAM int term and returns a c_long value. Should only be used for C interop with Zig functions. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_isize/2","title":"beam.get_isize/2","type":"function","doc":"Takes a BEAM int term and returns a isize value. Should only be used for C interop. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_usize/2","title":"beam.get_usize/2","type":"function","doc":"Takes a BEAM int term and returns a usize value. Zig idiomatically uses usize for its size values, so typically you should be using this function. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_u8/2","title":"beam.get_u8/2","type":"function","doc":"Takes a BEAM int term and returns a u8 value. Note that this conversion function checks to make sure it&#39;s in range (0..255). Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_i32/2","title":"beam.get_i32/2","type":"function","doc":"Takes a BEAM int term and returns an i32 value. Note that this conversion function does not currently do range checking. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_i64/2","title":"beam.get_i64/2","type":"function","doc":"Takes a BEAM int term and returns an i64 value. Note that this conversion function does not currently do range checking. Raises beam.Error.FunctionClauseError if the term is not integer/0"},{"ref":"beam.html#get_f16/2","title":"beam.get_f16/2","type":"function","doc":"Takes a BEAM float term and returns an f16 value. Note that this conversion function does not currently do range checking. Raises beam.Error.FunctionClauseError if the term is not float/0"},{"ref":"beam.html#get_f32/2","title":"beam.get_f32/2","type":"function","doc":"Takes a BEAM float term and returns an f32 value. Note that this conversion function does not currently do range checking. Raises beam.Error.FunctionClauseError if the term is not float/0"},{"ref":"beam.html#get_f64/2","title":"beam.get_f64/2","type":"function","doc":"Takes a BEAM float term and returns an f64 value. Raises beam.Error.FunctionClauseError if the term is not float/0"},{"ref":"beam.html#get_atom_slice/2","title":"beam.get_atom_slice/2","type":"function","doc":"Takes a BEAM atom term and retrieves it as a slice []u8 value. it&#39;s the caller&#39;s responsibility to make sure that the value is freed. Uses the standard beam.allocator allocator. If you require a custom allocator, use get_atom_slice_alloc/3 Raises beam.Error.FunctionClauseError if the term is not atom/0"},{"ref":"beam.html#get_atom_slice_alloc/3","title":"beam.get_atom_slice_alloc/3","type":"function","doc":"Takes a BEAM atom term and retrieves it as a slice []u8 value, with any allocator. Raises beam.Error.FunctionClauseError if the term is not atom/0"},{"ref":"beam.html#get_c_string/2","title":"beam.get_c_string/2","type":"function","doc":"Takes an BEAM Kernel.binary/0 term and retrieves a pointer to the binary data as a Zig c-string ([*c]u8). No memory is allocated for this operation. Should only be used for c interop functions. Note: this function could have unexpected results if your BEAM binary contains any zero byte values. Always use get_char_slice/2 when C-interop is not necessary. Raises beam.Error.FunctionClauseError if the term is not Kernel.binary/0"},{"ref":"beam.html#get_char_slice/2","title":"beam.get_char_slice/2","type":"function","doc":"Takes an BEAM Kernel.binary/0 term and retrieves it as a Zig character slice ([]u8) No memory is allocated for this operation. Raises beam.Error.FunctionClauseError if the term is not Kernel.binary/0"},{"ref":"beam.html#get_binary/2","title":"beam.get_binary/2","type":"function","doc":"Takes an BEAM Kernel.binary/0 term and returns the corresponding binary struct. Raises beam.Error.FunctionClauseError if the term is not Kernel.binary/0"},{"ref":"beam.html#get_pid/2","title":"beam.get_pid/2","type":"function","doc":"Takes an BEAM Kernel.pid/0 term and returns the corresponding pid struct. Note that this is a fairly opaque struct and you&#39;re on your own as to what you can do with this (for now), except as a parameter for the e.ErlNifSend function. Raises beam.Error.FunctionClauseError if the term is not Kernel.pid/0"},{"ref":"beam.html#get_tuple/2","title":"beam.get_tuple/2","type":"function","doc":"Takes an Beam tuple/0 term and returns it as a slice of term structs. Does not allocate memory for this operation. Raises beam.Error.FunctionClauseError if the term is not tuple/0"},{"ref":"beam.html#get_list_length/2","title":"beam.get_list_length/2","type":"function","doc":"Takes a BEAM list/0 term and returns its length. Raises beam.Error.FunctionClauseError if the term is not list/0"},{"ref":"beam.html#get_head_and_iter/2","title":"beam.get_head_and_iter/2","type":"function","doc":"Iterates over a BEAM list/0. In this function, the list value will be modified to the tl of the BEAM list, and the return value will be the BEAM term. Raises beam.Error.FunctionClauseError if the term is not list/0"},{"ref":"beam.html#get_slice_of/3","title":"beam.get_slice_of/3","type":"function","doc":"A generic function which lets you convert a BEAM list/0 of homogeous type into a Zig slice. The resulting slice will be allocated using the beam allocator, with ownership passed to the caller. If you need to use a different allocator, use get_slice_of_alloc/4 Raises beam.Error.FunctionClauseError if the term is not list/0. Also raises beam.Error.FunctionClauseError if any of the terms is incompatible with the internal type supported internal types: c_int c_long isize usize u8 i32 i64 f16 f32 f64"},{"ref":"beam.html#get_slice_of_alloc/4","title":"beam.get_slice_of_alloc/4","type":"function","doc":"Converts an BEAM list/0 of homogenous type into a Zig slice, but using any allocator you wish. ownership is passed to the caller. Raises beam.Error.FunctionClauseError if the term is not list/0. Also raises beam.Error.FunctionClauseError if any of the terms is incompatible with the internal type. supported internal types: c_int c_long isize usize u8 i32 i64 f16 f32 f64"},{"ref":"beam.html#get_bool/2","title":"beam.get_bool/2","type":"function","doc":"Converts an BEAM boolean/0 into a Zig bool. Raises beam.Error.FunctionClauseError if the term is not boolean/0. May potentially raise an out of memory error, as it must make an allocation to perform its conversion."},{"ref":"beam.html#make/3","title":"beam.make/3","type":"function","doc":"A helper for marshalling values from Zig back into the runtime. Use this function if you need support for Zig generics. supported types: c_int c_long isize usize u8 i32 i64 f16 f32 f64"},{"ref":"beam.html#make_u8/2","title":"beam.make_u8/2","type":"function","doc":"converts a char (u8) value into a BEAM integer/0."},{"ref":"beam.html#make_c_int/2","title":"beam.make_c_int/2","type":"function","doc":"converts a c_int value into a BEAM integer/0."},{"ref":"beam.html#make_c_long/2","title":"beam.make_c_long/2","type":"function","doc":"converts a c_long value into a BEAM integer/0."},{"ref":"beam.html#make_isize/2","title":"beam.make_isize/2","type":"function","doc":"converts an isize value into a BEAM integer/0."},{"ref":"beam.html#make_usize/2","title":"beam.make_usize/2","type":"function","doc":"converts a usize value into a BEAM integer/0."},{"ref":"beam.html#make_i32/2","title":"beam.make_i32/2","type":"function","doc":"converts an i32 value into a BEAM integer/0."},{"ref":"beam.html#make_i64/2","title":"beam.make_i64/2","type":"function","doc":"converts an i64 value into a BEAM integer/0."},{"ref":"beam.html#make_f16/2","title":"beam.make_f16/2","type":"function","doc":"converts an f16 value into a BEAM float/0."},{"ref":"beam.html#make_f32/2","title":"beam.make_f32/2","type":"function","doc":"converts an f32 value into a BEAM float/0."},{"ref":"beam.html#make_f64/2","title":"beam.make_f64/2","type":"function","doc":"converts an f64 value into a BEAM float/0."},{"ref":"beam.html#make_atom/2","title":"beam.make_atom/2","type":"function","doc":"converts a Zig char slice ([]u8) into a BEAM atom/0."},{"ref":"beam.html#make_slice/2","title":"beam.make_slice/2","type":"function","doc":"converts a Zig char slice ([]u8) into a BEAM binary/0. no memory allocation inside of Zig is performed and the BEAM environment is responsible for the resulting binary. You are responsible for managing the allocation of the slice."},{"ref":"beam.html#make_c_string/2","title":"beam.make_c_string/2","type":"function","doc":"converts an c string ([*c]u8) into a BEAM binary/0. Mostly used for c interop. no memory allocation inside of Zig is performed and the BEAM environment is responsible for the resulting binary. You are responsible for managing the allocation of the slice."},{"ref":"beam.html#make_tuple/2","title":"beam.make_tuple/2","type":"function","doc":"converts a slice of terms into a BEAM tuple/0."},{"ref":"beam.html#make_term_list/2","title":"beam.make_term_list/2","type":"function","doc":"converts a slice of terms into a BEAM list/0."},{"ref":"beam.html#make_charlist/2","title":"beam.make_charlist/2","type":"function","doc":"converts a Zig char slice ([]u8) into a BEAM charlist/0."},{"ref":"beam.html#make_cstring_charlist/2","title":"beam.make_cstring_charlist/2","type":"function","doc":"converts a c string ([*c]u8) into a BEAM charlist/0."},{"ref":"beam.html#make_list/3","title":"beam.make_list/3","type":"function","doc":"A helper to make BEAM lists out of slices of term. Use this function if you need a generic listbuilding function. uses the BEAM allocator internally. If you would like to use a custom allocator, (for example an arena allocator, if you have very long lists), use make_list_alloc/4 supported internal types: c_int c_long isize usize u8 i32 i64 f16 f32 f64"},{"ref":"beam.html#make_list_alloc/4","title":"beam.make_list_alloc/4","type":"function","doc":"A helper to make a BEAM t:Kernel.list out of terms, with any allocator. Use this function if you need a generic listbuilding function. supported internal types: c_int c_long isize usize u8 i32 i64 f16 f32 f64"},{"ref":"beam.html#make_c_int_list/2","title":"beam.make_c_int_list/2","type":"function","doc":"converts a c_int slice ([]c_int) into a BEAM list of integer/0."},{"ref":"beam.html#make_c_long_list/2","title":"beam.make_c_long_list/2","type":"function","doc":"converts a c_long slice ([]c_long) into a BEAM list of integer/0."},{"ref":"beam.html#make_i32_list/2","title":"beam.make_i32_list/2","type":"function","doc":"converts an i32 slice ([]i32) into a BEAM list of integer/0."},{"ref":"beam.html#make_i64_list/2","title":"beam.make_i64_list/2","type":"function","doc":"converts an i64 slice ([]i64) into a BEAM list of integer/0."},{"ref":"beam.html#make_f16_list/2","title":"beam.make_f16_list/2","type":"function","doc":"converts an f16 slice ([]f16) into a BEAM list of float/0."},{"ref":"beam.html#make_f32_list/2","title":"beam.make_f32_list/2","type":"function","doc":"converts an f32 slice ([]f32) into a BEAM list of float/0."},{"ref":"beam.html#make_f64_list/2","title":"beam.make_f64_list/2","type":"function","doc":"converts an f64 slice ([]f64) into a BEAM list of float/0."},{"ref":"beam.html#make_bool/2","title":"beam.make_bool/2","type":"function","doc":"converts a bool value into a Kernel.boolean/0 value."},{"ref":"beam.html#make_ok_tuple/3","title":"beam.make_ok_tuple/3","type":"function","doc":"A helper to make {:ok, term} terms from arbitrarily-typed values. supported types: c_int c_long isize usize u8 i32 i64 f16 f32 f64 Use make_ok_term/2 to make ok tuples from generic terms. Use make_ok_atom/2 to make ok tuples with atom terms from slices."},{"ref":"beam.html#make_ok_binary/2","title":"beam.make_ok_binary/2","type":"function","doc":"A helper to make {:ok, binary} terms from slices"},{"ref":"beam.html#make_ok_atom/2","title":"beam.make_ok_atom/2","type":"function","doc":"A helper to make {:ok, atom} terms from slices"},{"ref":"beam.html#make_ok_term/2","title":"beam.make_ok_term/2","type":"function","doc":"A helper to make {:ok, term} terms in general"},{"ref":"beam.html#make_error_tuple/3","title":"beam.make_error_tuple/3","type":"function","doc":"A helper to make {:error, term} terms from arbitrarily-typed values. supported types: c_int c_long isize usize u8 i32 i64 f16 f32 f64 Use make_error_term/2 to make error tuples from generic terms. Use make_error_atom/2 to make atom errors from slices."},{"ref":"beam.html#make_error_atom/2","title":"beam.make_error_atom/2","type":"function","doc":"A helper to make {:error, atom} terms from slices"},{"ref":"beam.html#make_error_binary/2","title":"beam.make_error_binary/2","type":"function","doc":"A helper to make {:error, binary} terms from slices"},{"ref":"beam.html#make_error_term/2","title":"beam.make_error_term/2","type":"function","doc":"A helper to make {:error, term} terms in general"},{"ref":"beam.html#throw_enomem/1","title":"beam.throw_enomem/1","type":"function","doc":"This function is used to communicate :enomem back to the BEAM as an exception. The BEAM is potentially OOM-safe, and Zig lets you leverage that. OOM errors from beam.allocator can be converted to a generic erlang term that represents an exception. Returning this from your NIF results in a BEAM throw event."},{"ref":"beam.html#throw_function_clause_error/1","title":"beam.throw_function_clause_error/1","type":"function","doc":"This function is used to communicate :function_clause back to the BEAM as an exception. By default Zigler will do parameter input checking on value ingress from the dynamic BEAM runtime to the static Zig runtime. You can also use this function to communicate a similar error by returning the resulting term from your NIF."},{"ref":"beam.html#throw_assertion_error/1","title":"beam.throw_assertion_error/1","type":"function","doc":"This function is used to communicate :assertion_error back to the BEAM as an exception. Used when running Zigtests, when trapping beam.AssertionError.AssertionError."},{"ref":"beam.html#assert/1","title":"beam.assert/1","type":"function","doc":"A function used to return assertion errors to a zigtest. Zig&#39;s std.assert() will panic the Zig runtime and therefore the entire BEAM VM, making it incompatible with Elixir&#39;s Unit tests. As the VM is required for certain functionality (like e.enif_alloc), a BEAM-compatible assert is necessary. When building zigtests, assert(...) calls get lexically converted to try beam.assert(...) calls."},{"ref":"beam.AssertionError.AssertionError.html","title":"beam.AssertionError.AssertionError","type":"exception","doc":"Translates to ExUnit.AssertionError. Mostly used in Zig unit tests. All test clauses in the directories of your Zig-enabled modules are converted to Zig functions with the inferred type !void. The beam.assert/1 function can throw this error as its error type. Zigler converts assert statements in test blocks to try beam.assert(...);"},{"ref":"beam.Error.FunctionClauseError.html","title":"beam.Error.FunctionClauseError","type":"exception","doc":"Translates to Elixir FunctionClauseError. This is the default mechanism for reporting that a Zigler nif function has been incorrectly passed a value from the Elixir BEAM runtime. This is very important, as Zig is statically typed. support for users to be able to throw this value in their own Zig functions is forthcoming."},{"ref":"erl_nif.html","title":"erl_nif","type":"module","doc":"Provides erlang&#39;s NIF convenience functions to your Zig code. This struct derives from zig/beam/erl_nif.zig. Usually brought in as const e = @import(&quot;erl_nif.zig&quot;).c;, leading to the aliased shortcut e. for example, to call the enif_alloc function, you would use the following code: const e = @import(&quot;erl_nif.zig&quot;).c; pub fn give_me_ten_bytes() ?*u8 { return e.enif_alloc(10); } refer to the erlang documentation for available functions WARNING: currently, erl_nif.h is manually translated from the erlang header to be more digestible, and some functions available in the BEAM may not have been implemented yet, and windows support is currently not possible."},{"ref":"readme.html","title":"Zigler","type":"extras","doc":"Zigler Library test status: Dependent package test status:"},{"ref":"readme.html#installation","title":"Zigler - Installation","type":"extras","doc":"Zigler is available in Hex, and the package can be installed by adding zigler to your list of dependencies in mix.exs: def deps do [ {:zigler, &quot;~&gt; 0.1.3&quot;, runtime: false} ] end Documentation be found at https://hexdocs.pm/zigler. once you have this dependency, you should cache the zig build tools by running the following: mix zigler.get_zig latest"},{"ref":"readme.html#currently-supported-platforms","title":"Zigler - Currently supported platforms","type":"extras","doc":"Linux FreeBSD (tested, but not subjected to CI) MacOS (I believe it works but is still offically untested)"},{"ref":"readme.html#zig-nifs-made-easy","title":"Zigler - Zig Nifs made easy","type":"extras","doc":"Wouldn&#39;t it be nice if you could make NIFs as easily as you can use the asm keyword in C? This is now possible, using the magic of Zig. defmodule ExampleZig do use Zigler, app: :my_app ~Z&quot;&quot;&quot; /// nif: example_fun/2 fn example_fun(value1: f64, value2: f64) bool { return value1 &gt; value2; } &quot;&quot;&quot; end iex&gt; ExampleZig.example_fun(0.1, 0.4) false iex&gt; ExampleZig.example_fun(0.8, -0.8) true Zigler will do automatic type marshalling between Elixir code and Zig code. It will also convert trickier types into types you care about, for example: defmodule ZigCollections do use Zigler, app: :my_app ~Z&quot;&quot;&quot; /// nif: string_count/1 fn string_count(string: []u8) i64 { return @intCast(i64, string.len); } /// nif: list_sum/1 fn list_sum(array: []f64) f64 { var sum: f64 = 0.0; for(array) | item | { sum += item; } return sum; } &quot;&quot;&quot; end iex&gt; ZigCollections.string_count(&quot;hello zig&quot;) 9 iex&gt; ZigCollections.list_sum([1.0, 2.0, 3.0]) 6.0 Memory allocation with zigler is easy! A standard BEAM allocator is provided for you, so any zig code you import will play nice with the BEAM. defmodule Allocations do use Zigler, app: :my_app ~Z&quot;&quot;&quot; /// nif: double_atom/1 fn double_atom(env: beam.env, string: []u8) beam.atom { var double_string = beam.allocator.alloc(u8, string.len * 2) catch { return beam.throw_enomem(env); }; defer beam.allocator.free(double_string); for (string) | char, i | { double_string[i] = char; double_string[i + string.len] = char; } return beam.make_atom(env, double_string); } &quot;&quot;&quot; end iex&gt; Allocations.double_atom(&quot;foo&quot;) :foofoo It is a goal for Zigler to make using it to bind C libraries easier than using C to bind C libraries. Here is an example: defmodule BlasDynamic do use Zigler, app: :zigler, libs: [&quot;/usr/lib/x86_64-linux-gnu/blas/libblas.so&quot;], include: [&quot;/usr/include/x86_64-linux-gnu&quot;] ~Z&quot;&quot;&quot; const blas = @cImport({ @cInclude(&quot;cblas.h&quot;); }); /// nif: blas_axpy/3 fn blas_axpy(env: beam.env, a: f64, x: []f64, y: []f64) beam.term { if (x.len != y.len) { return beam.throw_function_clause_error(env); } blas.cblas_daxpy(@intCast(c_int, x.len), a, &amp;x[0], 1, &amp;y[0], 1); return beam.make_f64_list(env, y) catch { return beam.throw_function_clause_error(env); }; } &quot;&quot;&quot; end test &quot;we can use dynamically-linked blas&quot; do # returns aX+Y assert [11.0, 18.0] == BlasDynamic.blas_axpy(3.0, [2.0, 4.0], [5.0, 6.0]) end Zigler even has support for zig docstrings. defmodule AllTheDocs do use Zigler, app: :zigler ~Z&quot;&quot;&quot; /// a zero-arity function which returns 47. /// nif: zero_arity/0 fn zeroarity() i64 { return 47; } &quot;&quot;&quot; end iex&gt; h AllTheDocs.zeroarity def zeroarity() a zero-arity function which returns 47."}]